@@ -1,4 +1,4 @@
-# Copyright 2020 - 2024 Ternaris
+# Copyright 2020 - 2025 Ternaris
 # SPDX-License-Identifier: Apache-2.0
 """Rosbags support for rosbag2 files.
 
@@ -7,13 +7,17 @@ in the rosbag2 format.
 
 """
 
-from .errors import ReaderError
+from .enums import CompressionFormat, CompressionMode, StoragePlugin
+from .errors import ReaderError, WriterError
 from .reader import Reader
-from .writer import Writer, WriterError
+from .writer import Writer
 
 __all__ = [
+    'CompressionFormat',
+    'CompressionMode',
     'Reader',
     'ReaderError',
+    'StoragePlugin',
     'Writer',
     'WriterError',
 ]
