@@ -2,8 +2,6 @@
 # SPDX-License-Identifier: Apache-2.0
 """Rosbag2 reader."""
 
-# pyright: strict, reportUnreachable=false
-
 from __future__ import annotations
 
 from pathlib import Path
@@ -21,9 +19,9 @@ from rosbags.interfaces import (
     MessageDefinitionFormat,
     TopicInfo,
 )
-from rosbags.rosbag2.metadata import parse_qos
 
 from .errors import ReaderError
+from .metadata import ReaderMetadata, parse_qos
 from .storage_mcap import McapReader
 from .storage_sqlite3 import Sqlite3Reader
 
@@ -40,10 +38,13 @@ if TYPE_CHECKING:
 
     from .metadata import FileInformation, Metadata
 
-    class StorageReader(Protocol):
-        """Storage Protocol."""
+    class ReaderProtocol(Protocol):
+        """Sub Reader Protocol."""
 
-        def __init__(self, paths: Iterable[Path], connections: Iterable[Connection]) -> None:
+        connections: list[Connection]
+        metadata: ReaderMetadata
+
+        def __init__(self, path: Path) -> None:
             """Initialize."""
             raise NotImplementedError  # pragma: no cover
 
@@ -55,13 +56,9 @@ if TYPE_CHECKING:
             """Close file."""
             raise NotImplementedError  # pragma: no cover
 
-        def get_definitions(self) -> dict[str, MessageDefinition]:
-            """Get message definitions."""
-            raise NotImplementedError  # pragma: no cover
-
         def messages(
             self,
-            connections: Iterable[Connection] = (),
+            connections: Iterable[Connection],
             start: int | None = None,
             stop: int | None = None,
         ) -> Generator[tuple[Connection, int, bytes], None, None]:
@@ -69,8 +66,8 @@ if TYPE_CHECKING:
             raise NotImplementedError  # pragma: no cover
 
 
-class Reader:
-    """Reader for rosbag2 files.
+class DirectoryReader:
+    """Reader for rosbag2 directories with metadata.yaml file.
 
     It implements all necessary features to access metadata and message
     streams.
@@ -89,12 +86,12 @@ class Reader:
 
     """
 
-    STORAGE_PLUGINS: Mapping[str, type[StorageReader]] = {
+    STORAGE_PLUGINS: Mapping[str, type[ReaderProtocol]] = {
         'mcap': McapReader,
         'sqlite3': Sqlite3Reader,
     }
 
-    def __init__(self, path: Path | str) -> None:
+    def __init__(self, path: Path) -> None:
         """Open rosbag and check metadata.
 
         Args:
@@ -104,15 +101,28 @@ class Reader:
             ReaderError: Bag not readable or bag metadata.
 
         """
-        path = Path(path)
-        yamlpath = path / 'metadata.yaml'
         self.path = path
+        if not (path / 'metadata.yaml').exists():
+            msg = f'Expected metadata file {str(self.path)!r} does not exist.'
+            raise FileNotFoundError(msg)
+
+        self.tmpdir: TemporaryDirectory[str] | None = None
+        self.connections: list[Connection] = []
+        self.metadata = ReaderMetadata(0, 2**63 - 1, 0, 0, None, None, None, None)
+        self.files: list[FileInformation] = []
+        self.storages: list[ReaderProtocol] = []
+
+    def open(self) -> None:
+        """Open rosbag2."""
+        yamlpath = self.path / 'metadata.yaml'
         try:
             yaml = YAML(typ='safe')
             dct = cast(
                 'dict[str, Metadata]',
-                yaml.load(yamlpath.read_text()),  # pyright: ignore[reportUnknownMemberType]
+                yaml.load(yamlpath.read_text()),
             )
+        except PermissionError:
+            raise
         except OSError as err:
             msg = f'Could not read metadata at {yamlpath}: {err}.'
             raise ReaderError(msg) from None
@@ -121,17 +131,26 @@ class Reader:
             raise ReaderError(msg) from None
 
         try:
-            self.metadata: Metadata = dct['rosbag2_bagfile_information']
-            if (ver := self.metadata['version']) > 9:
+            metadata: Metadata = dct['rosbag2_bagfile_information']
+            if (ver := metadata['version']) > 9:
                 msg = f'Rosbag2 version {ver} not supported; please report issue.'
                 raise ReaderError(msg)
-            if (storageid := self.metadata['storage_identifier']) not in self.STORAGE_PLUGINS:
+
+            paths = [self.path / Path(x).name for x in metadata['relative_file_paths']]
+            if missing := [x for x in paths if not x.exists()]:
+                msg = f'Some database files are missing: {[str(x) for x in missing]!r}'
+                raise ReaderError(msg)
+
+            if (storageid := metadata['storage_identifier']) not in self.STORAGE_PLUGINS:
                 msg = f'Storage plugin {storageid!r} not supported; please report issue.'
                 raise ReaderError(msg)
 
-            self.paths = [path / Path(x).name for x in self.metadata['relative_file_paths']]
-            if missing := [x for x in self.paths if not x.exists()]:
-                msg = f'Some database files are missing: {[str(x) for x in missing]!r}'
+            compression_format = metadata.get('compression_format', None) or None
+            mode = metadata.get('compression_mode', '').lower()
+            compression_mode = mode if mode != 'none' else None
+
+            if compression_mode and (compression_format) != 'zstd':
+                msg = f'Compression format {compression_format!r} is not supported.'
                 raise ReaderError(msg)
 
             self.connections = [
@@ -150,117 +169,227 @@ class Reader:
                     ),
                     owner=self,
                 )
-                for idx, x in enumerate(self.metadata['topics_with_message_count'])
+                for idx, x in enumerate(metadata['topics_with_message_count'])
             ]
-            noncdr = {
+            if noncdr := {
                 fmt
                 for x in self.connections
-                if isinstance(x.ext, ConnectionExtRosbag2)
-                if (fmt := x.ext.serialization_format) != 'cdr'
-            }
-            if noncdr:
+                if (fmt := cast('ConnectionExtRosbag2', x.ext).serialization_format) != 'cdr'
+            }:
                 msg = f'Serialization format {noncdr!r} is not supported.'
                 raise ReaderError(msg)
 
-            if self.compression_mode and (cfmt := self.compression_format) != 'zstd':
-                msg = f'Compression format {cfmt!r} is not supported.'
-                raise ReaderError(msg)
+            duration = metadata['duration']['nanoseconds']
+            start_time = metadata['starting_time']['nanoseconds_since_epoch']
+            message_count = metadata['message_count']
 
-            self.files: list[FileInformation] = self.metadata.get('files', [])[:]
-            self.custom_data: dict[str, str] | None = self.metadata.get('custom_data')
-
-            self.tmpdir: TemporaryDirectory[str] | None = None
-            self.storage: StorageReader | None = None
         except KeyError as exc:
             msg = f'A metadata key is missing {exc!r}.'
             raise ReaderError(msg) from None
 
-    @property
-    def duration(self) -> int:
-        """Duration in nanoseconds between earliest and latest messages."""
-        nsecs: int = self.metadata['duration']['nanoseconds']
-        return nsecs + 1 if self.message_count else 0
+        self.metadata = ReaderMetadata(
+            duration=duration + 1 if message_count else 0,
+            start_time=start_time if message_count else 2**63 - 1,
+            end_time=start_time + duration + 1 if message_count else 0,
+            message_count=message_count,
+            compression_format=compression_format,
+            compression_mode=compression_mode,
+            ros_distro=metadata.get('ros_distro'),
+            custom=metadata.get('custom_data'),
+        )
 
-    @property
-    def start_time(self) -> int:
-        """Timestamp in nanoseconds of the earliest message."""
-        nsecs: int = self.metadata['starting_time']['nanoseconds_since_epoch']
-        return nsecs if self.message_count else 2**63 - 1
+        self.files = metadata.get('files', [])[:]
 
-    @property
-    def end_time(self) -> int:
-        """Timestamp in nanoseconds after the latest message."""
-        return self.start_time + self.duration if self.message_count else 0
-
-    @property
-    def message_count(self) -> int:
-        """Total message count."""
-        return self.metadata['message_count']
-
-    @property
-    def compression_format(self) -> str | None:
-        """Compression format."""
-        return self.metadata.get('compression_format', None) or None
-
-    @property
-    def compression_mode(self) -> str | None:
-        """Compression mode."""
-        mode = self.metadata.get('compression_mode', '').lower()
-        return mode if mode != 'none' else None
-
-    @property
-    def topics(self) -> dict[str, TopicInfo]:
-        """Topic information."""
-        return {x.topic: TopicInfo(x.msgtype, x.msgdef, x.msgcount, [x]) for x in self.connections}
-
-    @property
-    def ros_distro(self) -> str | None:
-        """ROS distribution."""
-        return self.metadata.get('ros_distro')
-
-    def open(self) -> None:
-        """Open rosbag2."""
         storage_paths: list[Path] = []
-        if self.compression_mode == 'file':
+        if compression_mode == 'file':
             self.tmpdir = TemporaryDirectory()
             tmpdir = self.tmpdir.name
             decomp = zstandard.ZstdDecompressor()
-            for path in self.paths:
+            for path in paths:
                 storage_file = Path(tmpdir, path.stem)
                 with path.open('rb') as infile, storage_file.open('wb') as outfile:
                     _ = decomp.copy_stream(infile, outfile)
                 storage_paths.append(storage_file)
         else:
-            storage_paths = self.paths[:]
+            storage_paths = paths[:]
+
+        plugin = self.STORAGE_PLUGINS[metadata['storage_identifier']]
+        try:
+            for path in storage_paths:
+                storage = plugin(path)
+                storage.open()
+                self.storages.append(storage)
+        except:
+            self.close()
+            raise
 
-        self.storage = self.STORAGE_PLUGINS[self.metadata['storage_identifier']](
-            storage_paths,
-            self.connections,
-        )
-        self.storage.open()
-        definitions = self.storage.get_definitions()
         for idx, conn in enumerate(self.connections):
-            if desc := definitions.get(conn.msgtype):
-                self.connections[idx] = Connection(
-                    id=conn.id,
-                    topic=conn.topic,
-                    msgtype=conn.msgtype,
-                    msgdef=desc,
-                    digest=conn.digest,
-                    msgcount=conn.msgcount,
-                    ext=conn.ext,
-                    owner=conn.owner,
-                )
+            if msgdef := next(
+                (
+                    y.msgdef
+                    for x in self.storages
+                    for y in x.connections
+                    if y.msgtype == conn.msgtype and y.msgdef.format != MessageDefinitionFormat.NONE
+                ),
+                None,
+            ):
+                self.connections[idx] = conn._replace(msgdef=msgdef)
 
     def close(self) -> None:
         """Close rosbag2."""
-        assert self.storage
-        self.storage.close()
-        self.storage = None
+        while self.storages:
+            self.storages.pop().close()
         if self.tmpdir:
             self.tmpdir.cleanup()
             self.tmpdir = None
 
+    def messages(
+        self,
+        connections: Iterable[Connection],
+        start: int | None = None,
+        stop: int | None = None,
+    ) -> Generator[tuple[Connection, int, bytes], None, None]:
+        """Read messages from bag.
+
+        Args:
+            connections: Iterable with connections to filter for. An empty
+                iterable disables filtering on connections.
+            start: Yield only messages at or after this timestamp (ns).
+            stop: Yield only messages before this timestamp (ns).
+
+        Yields:
+            tuples of connection, timestamp (ns), and rawdata.
+
+        Raises:
+            ReaderError: If reader was not opened.
+
+        """
+        topics = [x.topic for x in connections]
+        for storage in self.storages:
+            storage_conns = [x for x in storage.connections if x.topic in topics]
+            connmap = {
+                x.id: next(y for y in connections if x.topic == y.topic) for x in storage_conns
+            }
+            if self.metadata.compression_mode == 'message':
+                decomp = zstandard.ZstdDecompressor().decompress
+                for storage_conn, timestamp, data in storage.messages(storage_conns, start, stop):
+                    yield connmap[storage_conn.id], timestamp, decomp(data)
+            else:
+                for storage_conn, timestamp, data in storage.messages(storage_conns, start, stop):
+                    yield connmap[storage_conn.id], timestamp, data
+
+
+class Reader:
+    """Unified Reader.
+
+    It can read from rosbag2 directories or from raw storage files.
+
+    """
+
+    STORAGE_PLUGINS: Mapping[str, type[ReaderProtocol]] = {
+        'dir': DirectoryReader,
+        '.mcap': McapReader,
+        '.db3': Sqlite3Reader,
+    }
+
+    def __init__(self, path: str | Path) -> None:
+        """Initialize.
+
+        Args:
+            path: Filesystem path to bag.
+
+        Raises:
+            ReaderError: Path does not exist.
+
+        """
+        self.path = Path(path)
+        if not self.path.exists():
+            msg = f'File {str(self.path)!r} does not exist.'
+            raise FileNotFoundError(msg)
+
+        self.is_open = False
+
+        self.storage: ReaderProtocol
+        plugin_key = 'dir' if self.path.is_dir() else self.path.suffix
+        if plugin := self.STORAGE_PLUGINS.get(plugin_key):
+            self.storage = plugin(self.path)
+        else:
+            msg = f'Unrecognized storage format {self.path.suffix!r}'
+            raise ReaderError(msg)
+
+    def _check_open(self) -> None:
+        """Ensure reader is open."""
+        if not self.is_open:
+            msg = 'Rosbag is not open.'
+            raise ReaderError(msg)
+
+    @property
+    def duration(self) -> int:
+        """Duration in nanoseconds between earliest and latest messages."""
+        self._check_open()
+        return self.storage.metadata.duration
+
+    @property
+    def start_time(self) -> int:
+        """Timestamp in nanoseconds of the earliest message."""
+        self._check_open()
+        return self.storage.metadata.start_time
+
+    @property
+    def end_time(self) -> int:
+        """Timestamp in nanoseconds after the latest message."""
+        self._check_open()
+        return self.storage.metadata.end_time
+
+    @property
+    def message_count(self) -> int:
+        """Total message count."""
+        self._check_open()
+        return self.storage.metadata.message_count
+
+    @property
+    def compression_format(self) -> str | None:
+        """Compression format."""
+        self._check_open()
+        return self.storage.metadata.compression_format
+
+    @property
+    def compression_mode(self) -> str | None:
+        """Compression mode."""
+        self._check_open()
+        return self.storage.metadata.compression_mode
+
+    @property
+    def connections(self) -> list[Connection]:
+        """Topic information."""
+        self._check_open()
+        return self.storage.connections
+
+    @property
+    def topics(self) -> dict[str, TopicInfo]:
+        """Topic information."""
+        self._check_open()
+        return {
+            x.topic: TopicInfo(x.msgtype, x.msgdef, x.msgcount, [x])
+            for x in self.storage.connections
+        }
+
+    @property
+    def ros_distro(self) -> str | None:
+        """ROS distribution."""
+        self._check_open()
+        return self.storage.metadata.ros_distro
+
+    def open(self) -> None:
+        """Open rosbag2."""
+        self.storage.open()
+        self.is_open = True
+
+    def close(self) -> None:
+        """Open rosbag2."""
+        self.storage.close()
+        self.is_open = False
+
     def messages(
         self,
         connections: Iterable[Connection] = (),
@@ -282,16 +411,8 @@ class Reader:
             ReaderError: If reader was not opened.
 
         """
-        if not self.storage:
-            msg = 'Rosbag is not open.'
-            raise ReaderError(msg)
-
-        if self.compression_mode == 'message':
-            decomp = zstandard.ZstdDecompressor().decompress
-            for connection, timestamp, data in self.storage.messages(connections, start, stop):
-                yield connection, timestamp, decomp(data)
-        else:
-            yield from self.storage.messages(connections, start, stop)
+        self._check_open()
+        return self.storage.messages(connections or self.storage.connections, start, stop)
 
     def __enter__(self) -> Self:
         """Open rosbag2 when entering contextmanager."""
